@@ -2,7 +2,7 @@
    X interface to SPIM
    (Derived from an earlier work by Alan Siow.)
 
-   Copyright (C) 1990-2000 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-2004 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
 
    SPIM is distributed under the following conditions:
@@ -21,7 +21,7 @@
    PURPOSE. */
 
 
-/* $Header: /Software/SPIM/src/buttons.c 7     2/01/01 8:57p Larus $
+/* $Header: /Software/SPIM/src/buttons.c 16    3/21/04 2:05p Larus $
  */
 
 #include <X11/Intrinsic.h>
@@ -43,11 +43,12 @@
 #include <ctype.h>
 
 #include "spim.h"
+#include "string-stream.h"
 #include "spim-utils.h"
 #include "xspim.h"
 #include "inst.h"
-#include "mem.h"
 #include "reg.h"
+#include "mem.h"
 #include "scanner.h"
 #include "sym-tbl.h"
 #include "buttons.h"
@@ -55,7 +56,6 @@
 
 /* Local functions: */
 
-#ifdef __STDC__
 static void add_breakpoint_action (Widget w, XtPointer client_data,
 				   XtPointer call_data);
 static void add_clear_button (Widget parent);
@@ -85,7 +85,7 @@ static void load_prompt_destroyed (Widget w, XtPointer client_data,
 				   XtPointer call_data);
 static void list_breakpoint_action (Widget w, XtPointer client_data,
 				    XtPointer call_data);
-static void noop (void);
+static void noop ();
 static void parse_print_value (Widget w, XtPointer client_data,
 			       XtPointer call_data);
 static void parse_set_value (Widget w, XtPointer client_data,
@@ -138,52 +138,6 @@ static void step_prompt_destroyed (Widget w, XtPointer client_data,
 static void warp_to_second_dialog (Widget widget, XEvent *event,
 				   String *params, Cardinal *num_params);
 
-#else
-
-static void add_breakpoint_action ();
-static void add_clear_button ();
-static void add_mode_button ();
-static void add_print_button ();
-static void add_reload_button ();
-static void add_terminal_button ();
-static void breakpoint_prompt ();
-static void breakpoint_prompt_destroyed ();
-static void clear_program_state_action ();
-static void continue_action ();
-static void continue_prompt_destroyed ();
-static void delete_breakpoint_action ();
-static void destroy_popup_prompt ();
-static void help_action ();
-static void init_stack ();
-static void load_prompt ();
-static void load_prompt_destroyed ();
-static void list_breakpoint_action ();
-static void noop ();
-static void parse_print_value ();
-static void parse_set_value ();
-static Widget popup_two_field_dialog ();
-static void print_mem_prompt ();
-static void print_prompt_destroyed ();
-static void print_symbol_action ();
-static void quit_action ();
-static void quit_prompt ();
-static void quit_prompt_destroyed ();
-static void read_assm_file_action ();
-static void reload_action ();
-static void run_program_action ();
-static void run_prompt ();
-static void run_prompt_destroyed ();
-static void select_mode_action ();
-static void set_value_action ();
-static void set_value_prompt ();
-static void set_value_destroyed ();
-static void step_continue_action ();
-static void step_program_action ();
-static void step_prompt ();
-static void step_prompt_destroyed ();
-static void warp_to_second_dialog ();
-
-#endif
 
 /* Local variables: */
 
@@ -194,27 +148,15 @@ char *xspim_file_name = NULL;	/* Retain last file's name. */
 
 
 
-#ifdef __STDC__
 void
 record_file_name_for_prompt (char *name)
-#else
-void
-record_file_name_for_prompt (name)
-     char *name;
-#endif
 {
   xspim_file_name = str_copy (name);
 }
 
 
-#ifdef __STDC__
 void
 create_buttons (Widget parent)
-#else
-void
-create_buttons (parent)
-     Widget parent;
-#endif
 {
   Widget button;
   Widget command;
@@ -292,15 +234,8 @@ create_buttons (parent)
 
 static Widget quit_popup = NULL;
 
-#ifdef __STDC__
 static void
 quit_prompt (Widget button, XtPointer client_data, XtPointer call_data)
-#else
-static void
-quit_prompt (button, client_data, call_data)
-     Widget button;
-     XtPointer client_data, call_data;
-#endif
 {
   Widget parent, dialog;
   Arg args[10];
@@ -332,15 +267,8 @@ quit_prompt (button, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 static void
 quit_action (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-quit_action (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   Widget dialog = (Widget) client_data;
 
@@ -350,15 +278,8 @@ quit_action (w, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 static void
 quit_prompt_destroyed (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-quit_prompt_destroyed (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   quit_popup = NULL;
 }
@@ -368,15 +289,8 @@ quit_prompt_destroyed (w, client_data, call_data)
 
 static Widget load_popup = NULL;
 
-#ifdef __STDC__
 static void
 load_prompt (Widget button, XtPointer client_data, XtPointer call_data)
-#else
-static void
-load_prompt (button, client_data, call_data)
-     Widget button;
-     XtPointer client_data, call_data;
-#endif
 {
   Widget parent, dialog;
   Arg args[10];
@@ -412,36 +326,22 @@ load_prompt (button, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 static void
 read_assm_file_action (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-read_assm_file_action (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   Widget dialog = (Widget) client_data;
   String value = XawDialogGetValueString (dialog);
 
   free (xspim_file_name);
   xspim_file_name = str_copy (value);
-  read_file (value, 1);
+  read_file (value);
 
   destroy_popup_prompt (NULL, (XtPointer) dialog, (XtPointer) NULL);
 }
 
 
-#ifdef __STDC__
 static void
 load_prompt_destroyed (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-load_prompt_destroyed (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   load_popup = NULL;
 }
@@ -452,15 +352,8 @@ load_prompt_destroyed (w, client_data, call_data)
 static Widget run_popup = NULL;
 static Widget run_field1_text, run_field2_text;
 
-#ifdef __STDC__
 static void
 run_prompt (Widget button, XtPointer client_data, XtPointer call_data)
-#else
-static void
-run_prompt (button, client_data, call_data)
-     Widget button;
-     XtPointer client_data, call_data;
-#endif
 {
   char sa[20];
 
@@ -481,15 +374,8 @@ run_prompt (button, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 static void
 run_program_action (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-run_program_action (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   Arg args[10];
   String value1, value2;
@@ -511,14 +397,8 @@ run_program_action (w, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 static void
 init_stack (char *args)
-#else
-static void
-init_stack (args)
-     char *args;
-#endif
 {
   int argc = 0;
   char *argv[10000];
@@ -547,15 +427,8 @@ init_stack (args)
 }
 
 
-#ifdef __STDC__
 static void
 run_prompt_destroyed (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-run_prompt_destroyed (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   run_popup = NULL;
 }
@@ -568,15 +441,8 @@ static char *step_size = NULL;	/* Retain step size */
 static Widget step_popup = NULL;
 static Widget step_field1_text, step_field2_text;
 
-#ifdef __STDC__
 static void
 step_prompt (Widget button, XtPointer client_data, XtPointer call_data)
-#else
-static void
-step_prompt (button, client_data, call_data)
-     Widget button;
-     XtPointer client_data, call_data;
-#endif
 {
   if (step_popup == NULL)
     {
@@ -597,15 +463,8 @@ step_prompt (button, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 static void
 step_program_action (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-step_program_action (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   Arg args[10];
   String value1, value2;
@@ -630,15 +489,8 @@ step_program_action (w, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 static void
 step_continue_action (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-step_continue_action (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   Widget dialog = (Widget) client_data;
 
@@ -649,15 +501,8 @@ step_continue_action (w, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 static void
 step_prompt_destroyed (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-step_prompt_destroyed (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   step_popup = NULL;
 }
@@ -665,14 +510,8 @@ step_prompt_destroyed (w, client_data, call_data)
 
 /* *** Reload *** */
 
-#ifdef __STDC__
 static void
 add_reload_button (Widget parent)
-#else
-static void
-add_reload_button (parent)
-     Widget parent;
-#endif
 {
   Widget command, menu, entry;
   Arg args[2];
@@ -690,26 +529,17 @@ add_reload_button (parent)
 }
 
 
-#ifdef __STDC__
 static void
 reload_action (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-reload_action (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
-  int assem_or_exec = (int) client_data;
-
   if (xspim_file_name == NULL)
     return;
 
   write_output (message_out, "Memory and registers cleared\n\n");
-  initialize_world (load_trap_handler ? trap_file : NULL);
+  initialize_world (load_exception_handler ? exception_file_name : NULL);
   write_startup_message ();
   stack_initialized = 0;
-  read_file (xspim_file_name, assem_or_exec);
+  read_file (xspim_file_name);
 }
 
 
@@ -720,14 +550,8 @@ reload_action (w, client_data, call_data)
 #define CLEAR_MEM_REGS 2
 #define CLEAR_CONSOLE 3
 
-#ifdef __STDC__
 static void
 add_clear_button (Widget parent)
-#else
-static void
-add_clear_button (parent)
-     Widget parent;
-#endif
 {
   Widget command, menu, entry;
   Arg args[2];
@@ -755,16 +579,9 @@ add_clear_button (parent)
 }
 
 
-#ifdef __STDC__
 static void
 clear_program_state_action (Widget w, XtPointer client_data,
 			    XtPointer call_data)
-#else
-static void
-clear_program_state_action (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   int clear_op = (int) client_data;
 
@@ -777,7 +594,7 @@ clear_program_state_action (w, client_data, call_data)
 
     case CLEAR_MEM_REGS:
       write_output (message_out, "Memory and registers cleared\n\n");
-      initialize_world (load_trap_handler ? trap_file : NULL);
+      initialize_world (load_exception_handler ? exception_file_name : NULL);
       write_startup_message ();
       stack_initialized = 0;
       break;
@@ -801,15 +618,8 @@ clear_program_state_action (w, client_data, call_data)
 static Widget set_value_popup = NULL;
 static Widget set_field1_text, set_field2_text;
 
-#ifdef __STDC__
 static void
 set_value_prompt (Widget button, XtPointer client_data, XtPointer call_data)
-#else
-static void
-set_value_prompt (button, client_data, call_data)
-     Widget button;
-     XtPointer client_data, call_data;
-#endif
 {
   if (set_value_popup == NULL)
     {
@@ -827,15 +637,8 @@ set_value_prompt (button, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 static void
 parse_set_value (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-parse_set_value (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   Arg args[10];
   String value1, value2;
@@ -853,14 +656,8 @@ parse_set_value (w, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 static void
 set_value_action (char *location_str, char *value_str)
-#else
-static void
-set_value_action (location_str, value_str)
-     char *location_str, *value_str;
-#endif
 {
   unsigned long value;
   int reg_no;
@@ -887,11 +684,11 @@ set_value_action (location_str, value_str)
   else if (reg_no > 0)
     R[reg_no] = (reg_word)value;
   else if (streq (location_str, "Status") || streq (location_str, "status"))
-    Status_Reg = (reg_word)value;
+    CP0_Status = (reg_word)value;
   else if (streq (location_str, "PC") || streq (location_str, "pc"))
     PC = (reg_word)value;
   else if (streq (location_str, "EPC") | streq (location_str, "epc"))
-    EPC = (reg_word)value;
+    CP0_EPC = (reg_word)value;
   else
     {
       char *eptr;
@@ -902,21 +699,14 @@ set_value_action (location_str, value_str)
       if (eptr == location_str)
 	error ("Unknown register selected: %s\n", location_str);
       else
-	SET_MEM_WORD (addr, value);
+	set_mem_word (addr, value);
     }
   redisplay_data ();
 }
 
 
-#ifdef __STDC__
 static void
 set_value_destroyed (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-set_value_destroyed (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   set_value_popup = NULL;
 }
@@ -924,14 +714,8 @@ set_value_destroyed (w, client_data, call_data)
 
 /* *** Print *** */
 
-#ifdef __STDC__
 static void
 add_print_button (Widget parent)
-#else
-static void
-add_print_button (parent)
-     Widget parent;
-#endif
 {
   Widget command, menu, entry;
   Arg args[2];
@@ -955,15 +739,8 @@ add_print_button (parent)
 static Widget print_popup = NULL;
 static Widget print_field1_text, print_field2_text;
 
-#ifdef __STDC__
 static void
 print_mem_prompt (Widget button, XtPointer client_data, XtPointer call_data)
-#else
-static void
-print_mem_prompt (button, client_data, call_data)
-     Widget button;
-     XtPointer client_data, call_data;
-#endif
 {
   if (print_popup == NULL)
     {
@@ -982,15 +759,8 @@ print_mem_prompt (button, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 static void
 parse_print_value (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-parse_print_value (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   Arg args[10];
   String value1, value2;
@@ -1020,29 +790,15 @@ parse_print_value (w, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 static void
 print_symbol_action (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-print_symbol_action (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   print_symbols ();
 }
 
 
-#ifdef __STDC__
 static void
 print_prompt_destroyed (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-print_prompt_destroyed (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   print_popup = NULL;
 }
@@ -1054,15 +810,8 @@ static char *breakpoint_addr = NULL; /* Retain last breakpoint address */
 
 static Widget bkpt_popup = NULL;
 
-#ifdef __STDC__
 static void
 breakpoint_prompt (Widget button, XtPointer client_data, XtPointer call_data)
-#else
-static void
-breakpoint_prompt (button, client_data, call_data)
-     Widget button;
-     XtPointer client_data, call_data;
-#endif
 {
   Widget parent, dialog;
   Arg args[10];
@@ -1103,15 +852,8 @@ breakpoint_prompt (button, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 static void
 add_breakpoint_action (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-add_breakpoint_action (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   Widget dialog = (Widget) client_data;
   String value = XawDialogGetValueString (dialog);
@@ -1132,15 +874,8 @@ add_breakpoint_action (w, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 static void
 delete_breakpoint_action (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-delete_breakpoint_action (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   Widget dialog = (Widget) client_data;
   String value = XawDialogGetValueString (dialog);
@@ -1156,30 +891,16 @@ delete_breakpoint_action (w, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 static void
 list_breakpoint_action (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-list_breakpoint_action (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   list_breakpoints ();
 }
 
 
-#ifdef __STDC__
 static void
 breakpoint_prompt_destroyed (Widget w, XtPointer client_data,
 			     XtPointer call_data)
-#else
-static void
-breakpoint_prompt_destroyed (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   bkpt_popup = NULL;
 }
@@ -1187,19 +908,11 @@ breakpoint_prompt_destroyed (w, client_data, call_data)
 
 /* *** Help *** */
 
-#ifdef __STDC__
 static void
 help_action (Widget w, XtPointer ignore, XtPointer ignored)
-#else
-static void
-help_action (w, ignore, ignored)
-     Widget w;
-     XtPointer ignore;
-     XtPointer ignored;
-#endif
 {
   static char * msg = "\nSPIM is a MIPS R2000 simulator.\n\
-Copyright (C) 1990-2000 by James R. Larus, larus@cs.wisc.edu\n\n\
+Copyright (C) 1990-2004 by James R. Larus, larus@cs.wisc.edu\n\n\
 quit  -- Exit from the simulator\n\
 load -- Read a file into memory\n\
 run -- Execute a program\n\
@@ -1218,14 +931,8 @@ mode -- Set SPIM operating modes\n";
 
 /* *** Terminal *** */
 
-#ifdef __STDC__
 static void
 add_terminal_button (Widget parent)
-#else
-static void
-add_terminal_button (parent)
-     Widget parent;
-#endif
 {
   Widget command, menu, entry;
   Arg args[2];
@@ -1246,14 +953,8 @@ add_terminal_button (parent)
 
 static Widget modeCommand;
 
-#ifdef __STDC__
 static void
 add_mode_button (Widget parent)
-#else
-static void
-add_mode_button (parent)
-     Widget parent;
-#endif
 {
   Widget menu, entry;
   Arg args[2];
@@ -1284,15 +985,8 @@ add_mode_button (parent)
 }
 
 
-#ifdef __STDC__
 static void
 select_mode_action (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-select_mode_action (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   Arg args[10];
   String name = XtName (w);
@@ -1322,14 +1016,8 @@ select_mode_action (w, client_data, call_data)
 
 static Widget continue_popup = NULL;
 
-#ifdef __STDC__
 void
 continue_prompt (int interrupt_seen)
-#else
-void
-continue_prompt (interrupt_seen)
-     int interrupt_seen;
-#endif
 {
   Widget dialog;
   Arg args[10];
@@ -1364,15 +1052,8 @@ continue_prompt (interrupt_seen)
 }
 
 
-#ifdef __STDC__
 static void
 continue_action (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-continue_action (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   Widget dialog = (Widget) client_data;
 
@@ -1384,16 +1065,9 @@ continue_action (w, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 static void
 continue_prompt_destroyed (Widget w, XtPointer client_data,
 			   XtPointer call_data)
-#else
-static void
-continue_prompt_destroyed (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   continue_popup = NULL;
 }
@@ -1401,32 +1075,16 @@ continue_prompt_destroyed (w, client_data, call_data)
 
 
 
-#ifdef __STDC__
 void
 confirm (Widget widget, XEvent *event, String *params, Cardinal *num_params)
-#else
-void
-confirm (widget, event, params, num_params)
-     Widget widget;
-     XEvent *event;
-     String *params;
-     Cardinal *num_params;
-#endif
 {
   Widget dialog = XtParent (widget);
   (*confirmAction) (widget, (XtPointer) dialog, (XtPointer) NULL);
 }
 
 
-#ifdef __STDC__
 static void
 destroy_popup_prompt (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-static void
-destroy_popup_prompt (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   Widget popup = XtParent ((Widget) client_data);
 
@@ -1435,18 +1093,12 @@ destroy_popup_prompt (w, client_data, call_data)
 }
 
 
-#ifdef __STDC__
-static void
-noop (void)
-#else
 static void
 noop ()
-#endif
 {
 }
 
 
-#ifdef __STDC__
 static Widget
 popup_two_field_dialog (Widget button, String name, String field1_label,
 			String field1_value, String field2_label,
@@ -1454,17 +1106,6 @@ popup_two_field_dialog (Widget button, String name, String field1_label,
 			void (*action) (/* ??? */), String action2_name,
 			void (*action2) (/* ??? */), Widget *field1_text,
 			Widget *field2_text)
-#else
-static Widget
-popup_two_field_dialog (button, name, field1_label, field1_value,
-		     field2_label, field2_value, action_name, action,
-		     action2_name, action2, field1_text, field2_text)
-     Widget button;
-     String name, field1_label, field1_value, field2_label, field2_value,
-       action_name, action2_name;
-     void (*action) (), (*action2) ();
-     Widget *field1_text, *field2_text;
-#endif
 {
   Widget popup, form;
   Widget labelx, field1, field2;
@@ -1548,18 +1189,9 @@ popup_two_field_dialog (button, name, field1_label, field1_value,
 }
 
 
-#ifdef __STDC__
 static void
 warp_to_second_dialog (Widget widget, XEvent *event, String *params,
 		       Cardinal *num_params)
-#else
-static void
-warp_to_second_dialog (widget, event, params, num_params)
-     Widget widget;
-     XEvent *event;
-     String *params;
-     Cardinal *num_params;
-#endif
 {
   Widget form = XtParent (widget);
   Widget second_dialog;
