@@ -1,7 +1,7 @@
 /* SPIM S20 MIPS simulator.
    Interface to code to read a MIPS a.out file.
 
-   Copyright (C) 1990-2000 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-2004 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
 
    SPIM is distributed under the following conditions:
@@ -20,17 +20,13 @@
    PURPOSE. */
 
 
-/* $Header: /Software/SPIM/src/read-aout.h 4     12/24/00 1:37p Larus $
+/* $Header: /Software/SPIM/src/read-aout.h 6     2/15/04 1:27p Larus $
 */
 
 
 /* Exported functions: */
 
-#ifdef __STDC__
 int read_aout_file (char *file_name);
-#else
-int read_aout_file ();
-#endif
 
 
 /* Exported Variables: */
