@@ -1,7 +1,7 @@
 /* SPIM S20 MIPS simulator.
    System calls implemented by simulator.
 
-   Copyright (C) 1990-2000 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-2004 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
 
    SPIM is distributed under the following conditions:
@@ -20,7 +20,7 @@
    PURPOSE. */
 
 
-/* $Header: /Software/SPIM/src/spim-syscall.h 4     12/24/00 1:37p Larus $
+/* $Header: /Software/SPIM/src/spim-syscall.h 7     2/15/04 1:07p Larus $
 */
 
 
@@ -34,4 +34,12 @@
 #define READ_STRING_SYSCALL	8
 #define SBRK_SYSCALL		9
 #define EXIT_SYSCALL		10
+#define PRINT_CHARACTER_SYSCALL	11
+#define READ_CHARACTER_SYSCALL	12
 
+#define OPEN_SYSCALL		13
+#define READ_SYSCALL		14
+#define WRITE_SYSCALL		15
+#define CLOSE_SYSCALL		16
+
+#define EXIT2_SYSCALL		17
