@@ -1,7 +1,7 @@
 /* SPIM S20 MIPS simulator.
    Misc. routines for SPIM.
 
-   Copyright (C) 1990-2003 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-2004 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
    Changes for DOS and Windows versions by David A. Carley (dac@cs.wisc.edu)
 
@@ -21,25 +21,22 @@
    PURPOSE. */
 
 
-/* $Header: /Software/SPIM/src/spim-utils.c 10    1/05/02 1:27p Larus $
+/* $Header: /Software/SPIM/src/spim-utils.c 25    9/26/04 10:39a Larus $
 */
 
 
 #include <stdio.h>
 #include <ctype.h>
 #include <string.h>
-#ifdef __STDC__
 #include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
 
 #include "spim.h"
+#include "string-stream.h"
 #include "spim-utils.h"
 #include "inst.h"
 #include "data.h"
-#include "mem.h"
 #include "reg.h"
+#include "mem.h"
 #include "scanner.h"
 #include "parser.h"
 #include "y.tab.h"
@@ -49,15 +46,9 @@
 
 /* Internal functions: */
 
-#ifdef __STDC__
 static mem_addr copy_int_to_stack (int n);
 static mem_addr copy_str_to_stack (char *s);
-static void delete_all_breakpoints (void);
-#else
-static mem_addr copy_int_to_stack ();
-static mem_addr copy_str_to_stack ();
 static void delete_all_breakpoints ();
-#endif
 
 
 int exception_occurred;
@@ -84,18 +75,12 @@ mem_addr initial_k_data_limit = K_DATA_LIMIT;
 
 /* Initialize or reinitialize the state of the machine. */
 
-#ifdef __STDC__
 void
-initialize_world (char* trap_file)
-#else
-void
-initialize_world (trap_file)
-     char* trap_file;
-#endif
+initialize_world (char* exception_file_name)
 {
   /* Allocate the floating point registers */
   if (FGR == NULL)
-    FPR = (double *) xmalloc (16 * sizeof (double));
+    FPR = (double *) xmalloc (FPR_LENGTH * sizeof (double));
   /* Allocate the memory */
   make_memory (initial_text_size,
 	       initial_data_size, initial_data_limit,
@@ -104,33 +89,30 @@ initialize_world (trap_file)
 	       initial_k_data_size, initial_k_data_limit);
   initialize_registers ();
   program_starting_address = 0;
+  initialize_inst_tables ();
   initialize_symbol_table ();
   k_text_begins_at_point (K_TEXT_BOT);
   k_data_begins_at_point (K_DATA_BOT);
   data_begins_at_point (DATA_BOT);
   text_begins_at_point (TEXT_BOT);
 
-#ifdef DJGPP
-  trap_file = (char*)searchpath(trap_file);
-#endif
-
-  if (trap_file != NULL)
+  if (exception_file_name != NULL)
     {
       int old_bare = bare_machine;
       int old_accept = accept_pseudo_insts;
 
-      bare_machine = 0;		/* Trap handler uses extended machine */
+      bare_machine = 0;		/* Exception handler uses extended machine */
       accept_pseudo_insts = 1;
-      if (read_assembly_file (trap_file))
-	fatal_error ("Cannot read trap handler: %s\n", trap_file);
+      if (read_assembly_file (exception_file_name))
+	fatal_error ("Cannot read exception handler: %s\n", exception_file_name);
       bare_machine = old_bare;
       accept_pseudo_insts = old_accept;
-      write_output (message_out, "Loaded: %s\n", trap_file);
+      write_output (message_out, "Loaded: %s\n", exception_file_name);
 
       if (!bare_machine)
       {
-	make_label_global ("main"); /* In case .globl main forgotten */
-	record_label ("main", 0, 0);
+	(void)make_label_global ("main"); /* In case .globl main forgotten */
+	(void)record_label ("main", 0, 0);
       }
     }
   initialize_scanner (stdin);
@@ -138,19 +120,14 @@ initialize_world (trap_file)
 }
 
 
-#ifdef __STDC__
-void
-write_startup_message (void)
-#else
 void
 write_startup_message ()
-#endif
 {
   write_output (message_out, "SPIM %s\n", SPIM_VERSION);
   write_output (message_out,
-		"Copyright 1990-2003 by James R. Larus (larus@cs.wisc.edu).\n");
+		"Copyright 1990-2004 by James R. Larus (larus@cs.wisc.edu).\n");
   write_output (message_out, "All Rights Reserved.\n");
-#if defined(DJGPP) || defined(WIN32)
+#ifdef WIN32
   write_output (message_out, "DOS and Windows ports by David A. Carley (dac@cs.wisc.edu).\n");
   write_output (message_out, "Copyright 1997 by Morgan Kaufmann Publishers, Inc.\n");
 #endif
@@ -159,45 +136,46 @@ write_startup_message ()
 
 
 
-#ifdef __STDC__
-void
-initialize_registers (void)
-#else
 void
 initialize_registers ()
-#endif
 {
-  memclr (FPR, 16 * sizeof (double));
+  memclr (FPR, FPR_LENGTH * sizeof (double));
   FGR = (float *) FPR;
   FWR = (int *) FPR;
-  memclr (R, 32 * sizeof (reg_word));
-  R[29] = STACK_TOP - BYTES_PER_WORD - 4096; /* Initialize $sp */
+
+  memclr (R, R_LENGTH * sizeof (reg_word));
+  R[REG_SP] = STACK_TOP - BYTES_PER_WORD - 4096; /* Initialize $sp */
   HI = LO = 0;
   PC = 0;
-  Cause = 0;
-  EPC = 0;
-  Status_Reg = 0;
-  BadVAddr = 0;
-  Context = 0;
-  PRId = 0;
+
+  CP0_BadVAddr = 0;
+  CP0_Count = 0;
+  CP0_Compare = 0;
+  CP0_Status = (CP0_Status_CU & 0x30000000) | CP0_Status_IM | CP0_Status_UM;
+  CP0_Cause = 0;
+  CP0_EPC = 0;
+#ifdef BIGENDIAN
+  CP0_Config =  CP0_Config_BE;
+#else
+  CP0_Config = 0;
+#endif
+
+  FIR = FIR_W | FIR_D | FIR_S;	/* Word, double, & single implemented */
+  FCSR = 0x0;
+  FCCR = 0x0;
+  FEXR = 0x0;
+  FENR = 0x0;
 }
 
 
 /* Read file NAME, which should contain assembly code. Return zero if
    successful and non-zero otherwise. */
 
-#ifdef __STDC__
 int
 read_assembly_file (char *name)
-#else
-int
-read_assembly_file (name)
-     char *name;
-#endif
 {
   FILE *file = fopen (name, "rt");
 
-  source_file = 1;
   if (file == NULL)
     {
       error ("Cannot open file: `%s'\n", name);
@@ -218,17 +196,12 @@ read_assembly_file (name)
 }
 
 
-#ifdef __STDC__
-mem_addr
-starting_address (void)
-#else
 mem_addr
 starting_address ()
-#endif
 {
   if (PC == 0)
     {
-      if (program_starting_address)
+      if (program_starting_address != 0)
 	return (program_starting_address);
       else
 	return (program_starting_address
@@ -245,15 +218,8 @@ starting_address ()
 #define environ	_environ
 #endif
 
-#ifdef __STDC__
 void
 initialize_run_stack (int argc, char **argv)
-#else
-void
-initialize_run_stack (argc, argv)
-     int argc;
-     char **argv;
-#endif
 {
   char **p;
   extern char **environ;
@@ -261,69 +227,64 @@ initialize_run_stack (argc, argv)
   mem_addr addrs[10000];
 
   /* Put strings on stack: */
-  for (p = environ; *p != '\0'; p++)
-    addrs[j++] = copy_str_to_stack (*p);
 
+  /* env: */
+  for (p = environ; *p != NULL; p++)
+    addrs[j++] = copy_str_to_stack (*p);
   env_j = j;
+
+  /* argv; */
   for (i = 0; i < argc; i++)
     addrs[j++] = copy_str_to_stack (argv[i]);
 
-  R[29] = R[29] & ~3;		/* Round down to nearest word */
-  R[29] -= BYTES_PER_WORD;	/* First free word on stack */
+  R[REG_SP] = STACK_TOP - BYTES_PER_WORD - 4096; /* Initialize $sp */
+  R[REG_SP] = R[REG_SP] & ~3;	/* Round down to nearest word */
+  R[REG_SP] -= BYTES_PER_WORD;	/* First free word on stack */
 
-  R[29] = R[29] & ~7;		/* Double-word align stack-pointer*/
+  R[REG_SP] = R[REG_SP] & ~7;	/* Double-word align stack-pointer*/
   if ((j % 2) != 0)		/* Odd number of arguments */
     {
-      R[29] -= BYTES_PER_WORD;	/* Ensure stack ends up double-word aligned */
+      R[REG_SP] -= BYTES_PER_WORD; /* Ensure stack ends up double-word aligned */
     }
 
   /* Build vectors on stack: */
-  copy_int_to_stack (0);	/* Null-terminate vector */
+
+  /* env: */
+  (void)copy_int_to_stack (0);	/* Null-terminate vector */
   for (i = env_j - 1; i >= 0; i--)
     R[REG_A2] = copy_int_to_stack (addrs[i]);
 
-  copy_int_to_stack (0);	/* Null-terminate vector */
+  /* argv: */
+  (void)copy_int_to_stack (0);	/* Null-terminate vector */
   for (i = j - 1; i >= env_j; i--)
     R[REG_A1] = copy_int_to_stack (addrs[i]);
 
+  /* argc: */
   R[REG_A0] = argc;
-
-  SET_MEM_WORD (R[29], argc);	/* Leave argc on stack */
+  set_mem_word (R[REG_SP], argc); /* Leave argc on stack */
 }
 
 
-#ifdef __STDC__
 static mem_addr
 copy_str_to_stack (char *s)
-#else
-static mem_addr
-copy_str_to_stack (s)
-     char *s;
-#endif
 {
   int i = strlen (s);
   while (i >= 0)
     {
-      SET_MEM_BYTE (R[29], s[i]);
-      R[29] -= 1;
+      set_mem_byte (R[REG_SP], s[i]);
+      R[REG_SP] -= 1;
       i -= 1;
     }
-  return ((mem_addr) R[29] + 1); /* Leaves stack pointer byte-aligned!! */
+  return ((mem_addr) R[REG_SP] + 1); /* Leaves stack pointer byte-aligned!! */
 }
 
 
-#ifdef __STDC__
 static mem_addr
 copy_int_to_stack (int n)
-#else
-static mem_addr
-copy_int_to_stack (n)
-     int n;
-#endif
 {
-  SET_MEM_WORD (R[29], n);
-  R[29] -= BYTES_PER_WORD;
-  return ((mem_addr) R[29] + BYTES_PER_WORD);
+  set_mem_word (R[REG_SP], n);
+  R[REG_SP] -= BYTES_PER_WORD;
+  return ((mem_addr) R[REG_SP] + BYTES_PER_WORD);
 }
 
 
@@ -332,15 +293,8 @@ copy_int_to_stack (n)
    non-zero, then step through a breakpoint.  Return non-zero if
    breakpoint is encountered. */
 
-#ifdef __STDC__
 int
 run_program (mem_addr pc, int steps, int display, int cont_bkpt)
-#else
-int
-run_program (pc, steps, display, cont_bkpt)
-     mem_addr pc;
-     int steps, display, cont_bkpt;
-#endif
 {
   if (cont_bkpt && inst_is_breakpoint (pc))
     {
@@ -356,10 +310,15 @@ run_program (pc, steps, display, cont_bkpt)
 
   exception_occurred = 0;
   if (!run_spim (pc, steps, display))
-    /* Can't restart program */
-    PC = 0;
-  if (exception_occurred && Cause == (BKPT_EXCPT << 2))
-    return (1);
+      /* Can't restart program */
+      PC = 0;
+  if (exception_occurred && CP0_ExCode == ExcCode_Bp)
+  {
+      /* Turn off EXL bit, so subsequent interrupts set EPC since the break is
+      handled by SPIM code, not MIPS code. */
+      CP0_Status &= ~CP0_Status_EXL;
+      return (1);
+  }
   else
     return (0);
 }
@@ -381,14 +340,8 @@ static bkpt *bkpts = NULL;
 
 /* Set a breakpoint at memory location ADDR. */
 
-#ifdef __STDC__
 void
 add_breakpoint (mem_addr addr)
-#else
-void
-add_breakpoint (addr)
-     mem_addr addr;
-#endif
 {
   bkpt *rec = (bkpt *) xmalloc (sizeof (bkpt));
 
@@ -410,14 +363,8 @@ add_breakpoint (addr)
 
 /* Delete all breakpoints at memory location ADDR. */
 
-#ifdef __STDC__
 void
 delete_breakpoint (mem_addr addr)
-#else
-void
-delete_breakpoint (addr)
-     mem_addr addr;
-#endif
 {
   bkpt *p, *b;
   int deleted_one = 0;
@@ -427,7 +374,7 @@ delete_breakpoint (addr)
       {
 	bkpt *n;
 
-	SET_MEM_INST (addr, b->inst);
+	set_mem_inst (addr, b->inst);
 	if (p == NULL)
 	  bkpts = b->next;
 	else
@@ -444,13 +391,8 @@ delete_breakpoint (addr)
 }
 
 
-#ifdef __STDC__
-static void
-delete_all_breakpoints (void)
-#else
 static void
 delete_all_breakpoints ()
-#endif
 {
   bkpt *b, *n;
 
@@ -465,13 +407,8 @@ delete_all_breakpoints ()
 
 /* List all breakpoints. */
 
-#ifdef __STDC__
-void
-list_breakpoints (void)
-#else
 void
 list_breakpoints ()
-#endif
 {
   bkpt *b;
 
@@ -486,60 +423,21 @@ list_breakpoints ()
 
 /* Utility routines */
 
-/* Print the error message then exit. */
 
-#ifndef WIN32
-/*VARARGS0*/
-#ifdef __STDC__
-void
-fatal_error (char *fmt, ...)
-#else
-void
-fatal_error (va_alist)
-va_dcl
-#endif
-{
-  va_list args;
-#ifdef __STDC__
-  va_start (args, fmt);
-#else
-  char *fmt;
-
-  va_start (args);
-#endif
-  fmt = va_arg (args, char *);
-
-#ifdef NO_VFPRINTF
-  _doprnt (fmt, args, stderr);
-#else
-  vfprintf (stderr, fmt, args);
-#endif
-  exit (-1);
-  /*NOTREACHED*/
-}
-#endif
-
-/* Return the entry in the hash TABLE of length LENGTH with key STRING.
+/* Return the entry in the linear TABLE of length LENGTH with key STRING.
+   TABLE must be sorted on the key field.
    Return NULL if no such entry exists. */
 
-#ifdef __STDC__
-inst_info *
-map_string_to_inst_info (inst_info tbl[], int tbl_len, char *id)
-#else
-inst_info *
-map_string_to_inst_info (tbl, tbl_len, id)
-     register inst_info tbl [];
-     int tbl_len;
-     register char *id;
-#endif
+name_val_val *
+map_string_to_name_val_val (name_val_val tbl[], int tbl_len, char *id)
 {
-  register int low = 0;
-  register int hi = tbl_len - 1;
+  int low = 0;
+  int hi = tbl_len - 1;
 
   while (low <= hi)
     {
-      register int mid = (low + hi) / 2;
-      register char *idp = id, *np = tbl[mid].name;
+      int mid = (low + hi) / 2;
+      char *idp = id, *np = tbl[mid].name;
 
       while (*idp == *np && *idp != '\0') {idp ++; np ++;}
 
@@ -555,26 +453,19 @@ map_string_to_inst_info (tbl, tbl_len, id)
 }
 
 
-/* Return the entry in the hash TABLE of length LENGTH with VALUE1 field NUM.
+/* Return the entry in the linear TABLE of length LENGTH with VALUE1 field NUM.
+   TABLE must be sorted on the VALUE1 field.
    Return NULL if no such entry exists. */
 
-#ifdef __STDC__
-inst_info *
-map_int_to_inst_info (inst_info tbl[], int tbl_len, int num)
-#else
-inst_info *
-map_int_to_inst_info (tbl, tbl_len, num)
-     register inst_info tbl [];
-     int tbl_len;
-     register int num;
-#endif
+name_val_val *
+map_int_to_name_val_val (name_val_val tbl[], int tbl_len, int num)
 {
-  register int low = 0;
-  register int hi = tbl_len - 1;
+  int low = 0;
+  int hi = tbl_len - 1;
 
   while (low <= hi)
     {
-      register int mid = (low + hi) / 2;
+      int mid = (low + hi) / 2;
 
       if (tbl[mid].value1 == num)
 	return (&tbl[mid]);
@@ -591,7 +482,7 @@ map_int_to_inst_info (tbl, tbl_len, num)
 #ifdef NEED_VSPRINTF
 char *
 vsprintf (str, fmt, args)
-     char *str, *fmt;
+     char *str,*fmt;
      va_list *args;
 {
   FILE _strbuf;
@@ -607,15 +498,8 @@ vsprintf (str, fmt, args)
 
 
 #ifdef NEED_STRTOL
-#ifdef __STDC__
 unsigned long
 strtol (const char* str, const char** eptr, int base)
-#else
-long
-strtol (str, eptr, base)
-     char *str, **eptr;
-     int base;
-#endif
 {
   long result;
 
@@ -639,15 +523,8 @@ strtol (str, eptr, base)
 #endif
 
 #ifdef NEED_STRTOUL
-#ifdef __STDC__
 unsigned long
 strtoul (const char* str, char** eptr, int base)
-#else
-unsigned long
-strtoul (str, eptr, base)
-     char *str, **eptr;
-     int base;
-#endif
 {
   unsigned long result;
 
@@ -671,33 +548,17 @@ strtoul (str, eptr, base)
 #endif
 
 
-#ifdef __STDC__
 char *
 str_copy (char *str)
-#else
-char *
-str_copy (str)
-     char *str;
-#endif
 {
   return (strcpy (xmalloc (strlen (str) + 1), str));
 }
 
 
-#ifdef __STDC__
 void *
 xmalloc (int size)
-#else
-char *
-xmalloc (size)
-int size;
-#endif
 {
-#ifdef __STDC__
   void *x = (void *) malloc (size);
-#else
-  char *x = (char *) malloc (size);
-#endif
 
   if (x == 0)
     fatal_error ("Out of memory at request for %d bytes.\n");
@@ -707,20 +568,10 @@ int size;
 
 /* Allocate a zero'ed block of storage. */
 
-#ifdef __STDC__
 void *
 zmalloc (int size)
-#else
-char *
-zmalloc (size)
-int size;
-#endif
 {
-#ifdef __STDC__
   void *z = (void *) malloc (size);
-#else
-  char *z = (char *) malloc (size);
-#endif
 
   if (z == 0)
     fatal_error ("Out of memory at request for %d bytes.\n");
