@@ -1,7 +1,7 @@
 /* SPIM S20 MIPS simulator.
    Terminal interface for SPIM simulator.
 
-   Copyright (C) 1990-2003 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-2004 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
    Changes for DOS and Windows versions by David A. Carley (dac@cs.wisc.edu)
 
@@ -21,14 +21,20 @@
    PURPOSE. */
 
 
-/* $Header: /Software/SPIM/src/spim.c 12    1/04/03 3:31p Larus $
+/* $Header: /Software/SPIM/src/spim.c 36    10/18/04 9:31p Larus $
 */
 
 
+#ifndef WIN32
+#include <unistd.h>
+#endif
 #include <stdio.h>
 #include <ctype.h>
 #include <setjmp.h>
 #include <signal.h>
+#include <arpa/inet.h>
+#include <sys/ioctl.h>
+
 
 #ifdef RS
 /* This is problem on HP Snakes, which define RS in syscall.h */
@@ -36,27 +42,18 @@
 #endif
 
 #include <sys/types.h>
+#include <sys/select.h>
 
 #ifdef _AIX
 #ifndef NBBY
 #define NBBY 8
 #endif
-#include <sys/select.h>
-#endif
-
-#ifdef DJGPP
-#define USE_TERMIO
-#define termio termios
 #endif
 
 
 #ifndef WIN32
 #include <sys/time.h>
-
-#ifdef USE_TERMIO
-#ifndef DJGPP
-#include <termio.h>
-#endif
+#ifdef USE_TERMIOS
 #include <termios.h>
 #else
 #include <sys/ioctl.h>
@@ -64,17 +61,14 @@
 #endif
 #endif
 
-#ifdef __STDC__
 #include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
 
 #include "spim.h"
+#include "string-stream.h"
 #include "spim-utils.h"
 #include "inst.h"
-#include "mem.h"
 #include "reg.h"
+#include "mem.h"
 #include "parser.h"
 #include "sym-tbl.h"
 #include "scanner.h"
@@ -83,35 +77,19 @@
 
 /* Internal functions: */
 
-#ifdef __STDC__
-static void console_to_program (void);
-static void console_to_spim (void);
-static void flush_to_newline (void);
-static int get_opt_int (void);
+static void console_to_program ();
+static void console_to_spim ();
+static void flush_to_newline ();
+static int get_opt_int ();
 static int parse_spim_command (FILE *file, int redo);
 static int print_reg (int reg_no);
 static int print_fp_reg (int reg_no);
 static int print_reg_from_string (char *reg);
 static void print_all_regs (int hex_flag);
-static int read_assembly_command (void);
-static int str_prefix (char *s1, char *s2, int min_match);
-static void top_level (void);
-static int read_token (void);
-#else
-static void console_to_program ();
-static void console_to_spim ();
-static void flush_to_newline ();
-static int get_opt_int ();
-static int parse_spim_command ();
-static int print_reg ();
-static int print_fp_reg ();
-static int print_reg_from_string ();
-static void print_all_regs ();
 static int read_assembly_command ();
-static int str_prefix ();
+static int str_prefix (char *s1, char *s2, int min_match);
 static void top_level ();
 static int read_token ();
-#endif
 
 
 /* Exported Variables: */
@@ -124,39 +102,33 @@ int delayed_branches;		/* Non-Zero => simulate delayed branches */
 int delayed_loads;		/* Non-Zero => simulate delayed loads */
 int accept_pseudo_insts;	/* Non-Zero => parse pseudo instructions  */
 int quiet;			/* Non-Zero => no warning messages */
-int source_file;		/* Non-Zero => program is source, not binary */
 port message_out, console_out, console_in;
 int mapped_io;			/* Non-zero => activate memory-mapped IO */
 int pipe_out;
-int cycle_level;		/* Non-zero => cycle level mode */
-
+int spim_return_value;		/* Value returned when spim exits */
 
 /* Local variables: */
 
-static int load_trap_handler = 1; /* Non-zero => load standard trap handler */
-char *trap_file = DEFAULT_TRAP_HANDLER;
+/* Non-zero => load standard exception handler */
+static int load_exception_handler = 1;
+static char *exception_file_name = DEFAULT_EXCEPTION_HANDLER;
 static int console_state_saved;
-#ifdef USE_TERMIO
-static struct termio saved_console_state;
+#ifdef USE_TERMIOS
+static struct termios saved_console_state;
 #else
 static struct sgttyb saved_console_state;
 #endif
+static int program_argc;
+static char** program_argv;
 
 
 
-#ifdef __STDC__
 int
 main (int argc, char **argv)
-#else
-int
-main (argc, argv)
-     int argc;
-     char **argv;
-#endif
 {
   int i;
-  int assembly_file_read = 0;
-  int argv_ptr = 0;
+  int assembly_file_loaded = 0;
+  int print_usage_msg = 0;
 
   console_out.f = stdout;
   message_out.f = stdout;
@@ -166,8 +138,7 @@ main (argc, argv)
   delayed_loads = 0;
   accept_pseudo_insts = 1;
   quiet = 0;
-  source_file = 0;
-  cycle_level = 0;
+  spim_return_value = 0;
 
   /* Input comes directly (not through stdio): */
   console_in.i = 0;
@@ -175,103 +146,163 @@ main (argc, argv)
 
   write_startup_message ();
 
-  if (argc == 2)
+  for (i = 1; i < argc; i++)
     {
-      /* Only one argument better be a file name. */
-      initialize_world (load_trap_handler ? trap_file : NULL);
-      assembly_file_read |= !read_assembly_file (argv[++i]);
-    }
-  else
-    for (i = 1; i < argc; i++)
-      {
 #ifdef WIN32
-	/* On Windows, support "/option" as well as "-option" */
-	if (argv [i][0] == '/')
-	  argv [i][0] = '-';
+      if (argv [i][0] == '/') { argv [i][0] = '-'; }
 #endif
-
-	if (streq (argv [i], "-bare"))
-	  {
-	    bare_machine = 1;
-	    delayed_branches = 1;
-	    delayed_loads = 1;
-	    quiet = 1;
-	  }
-	else if (streq (argv [i], "-asm"))
-	  {
-	    bare_machine = 0;
-	    delayed_branches = 0;
-	    delayed_loads = 0;
-	  }
-	else if (streq (argv [i], "-delayed_branches"))
-	  {
-	    delayed_branches = 1;
-	  }
-	else if (streq (argv [i], "-delayed_loads"))
-	  {
-	    delayed_loads = 1;
-	  }
-	else if (streq (argv [i], "-pseudo"))
-	  accept_pseudo_insts = 1;
-	else if (streq (argv [i], "-nopseudo"))
-	  accept_pseudo_insts = 0;
-	else if (streq (argv [i], "-trap"))
-	  load_trap_handler = 1;
-	else if (streq (argv [i], "-notrap"))
-	  load_trap_handler = 0;
-	else if (streq (argv [i], "-trap_file"))
-	  {
-	    trap_file = argv[++i];
-	    load_trap_handler = 1;
-	  }
-	else if (streq (argv [i], "-quiet"))
+      if (streq (argv [i], "-asm")
+	  || streq (argv [i], "-a"))
+	{
+	  bare_machine = 0;
+	  delayed_branches = 0;
+	  delayed_loads = 0;
+	}
+      else if (streq (argv [i], "-bare")
+	       || streq (argv [i], "-b"))
+	{
+	  bare_machine = 1;
+	  delayed_branches = 1;
+	  delayed_loads = 1;
 	  quiet = 1;
-	else if (streq (argv [i], "-noquiet"))
-	  quiet = 0;
-	else if (streq (argv [i], "-file"))
-	  {
-	    argv_ptr = i + 1;
-	    if (!assembly_file_read)
-	      {
-		initialize_world (load_trap_handler ? trap_file : NULL);
-	      }
-	    assembly_file_read |= !read_assembly_file (argv[++i]);
-	    break;			/* Everything that follows is argv */
-	  }
-	else if (streq (argv [i], "-mapped_io"))
-	  mapped_io = 1;
-	else if (streq (argv [i], "-nomapped_io"))
-	  mapped_io = 0;
-	else if (streq (argv [i], "-stext"))
-	  initial_text_size = atoi (argv[++i]);
-	else if (streq (argv [i], "-sdata"))
-	  initial_data_size = atoi (argv[++i]);
-	else if (streq (argv [i], "-ldata"))
-	  initial_data_limit = atoi (argv[++i]);
-	else if (streq (argv [i], "-sstack"))
-	  initial_stack_size = atoi (argv[++i]);
-	else if (streq (argv [i], "-lstack"))
-	  initial_stack_limit = atoi (argv[++i]);
+	}
+      else if (streq (argv [i], "-delayed_branches")
+	       || streq (argv [i], "-db"))
+	{
+	  delayed_branches = 1;
+	}
+      else if (streq (argv [i], "-delayed_loads")
+	       || streq (argv [i], "-dl"))
+	{
+	  delayed_loads = 1;
+	}
+      else if (streq (argv [i], "-exception")
+	       || streq (argv [i], "-e"))
+	{ load_exception_handler = 1; }
+      else if (streq (argv [i], "-noexception")
+	       || streq (argv [i], "-ne"))
+	{ load_exception_handler = 0; }
+      else if (streq (argv [i], "-exception_file")
+	       || streq (argv [i], "-ef"))
+	{
+	  exception_file_name = argv[++i];
+	  load_exception_handler = 1;
+	}
+      else if (streq (argv [i], "-mapped_io")
+	       || streq (argv [i], "-mio"))
+	{ mapped_io = 1; }
+      else if (streq (argv [i], "-nomapped_io")
+	       || streq (argv [i], "-nmio"))
+	{ mapped_io = 0; }
+      else if (streq (argv [i], "-pseudo")
+	       || streq (argv [i], "-p"))
+	{ accept_pseudo_insts = 1; }
+      else if (streq (argv [i], "-nopseudo")
+	       || streq (argv [i], "-np"))
+	{ accept_pseudo_insts = 0; }
+      else if (streq (argv [i], "-quiet")
+	       || streq (argv [i], "-q"))
+	{ quiet = 1; }
+      else if (streq (argv [i], "-noquiet")
+	       || streq (argv [i], "-nq"))
+	{ quiet = 0; }
+      else if (streq (argv [i], "-trap")
+	       || streq (argv [i], "-t"))
+	{ load_exception_handler = 1; }
+      else if (streq (argv [i], "-notrap")
+	       || streq (argv [i], "-nt"))
+	{ load_exception_handler = 0; }
+      else if (streq (argv [i], "-trap_file")
+	       || streq (argv [i], "-tf"))
+	{
+	  exception_file_name = argv[++i];
+	  load_exception_handler = 1;
+	}
+      else if (streq (argv [i], "-stext")
+	       || streq (argv [i], "-st"))
+	{ initial_text_size = atoi (argv[++i]); }
+      else if (streq (argv [i], "-sdata")
+	       || streq (argv [i], "-sd"))
+	{ initial_data_size = atoi (argv[++i]); }
+      else if (streq (argv [i], "-ldata")
+	       || streq (argv [i], "-ld"))
+	{ initial_data_limit = atoi (argv[++i]); }
+      else if (streq (argv [i], "-sstack")
+	       || streq (argv [i], "-ss"))
+	{ initial_stack_size = atoi (argv[++i]); }
+      else if (streq (argv [i], "-lstack")
+	       || streq (argv [i], "-ls"))
+	{ initial_stack_limit = atoi (argv[++i]); }
+      else if (streq (argv [i], "-sktext")
+	       || streq (argv [i], "-skt"))
+	{ initial_k_text_size = atoi (argv[++i]); }
+      else if (streq (argv [i], "-skdata")
+	       || streq (argv [i], "-skd"))
+	{ initial_k_data_size = atoi (argv[++i]); }
+      else if (streq (argv [i], "-lkdata")
+	       || streq (argv [i], "-lkd"))
+	{ initial_k_data_limit = atoi (argv[++i]); }
+      else if ((streq (argv [i], "-file")
+		|| streq (argv [i], "-f"))
+	       && (i + 1 < argc))
+	{
+	  program_argc = argc - (i + 1);
+	  program_argv = &argv[i + 1]; /* Everything following is argv */
 
-	else if (streq (argv [i], "-sktext"))
-	  initial_k_text_size = atoi (argv[++i]);
-	else if (streq (argv [i], "-skdata"))
-	  initial_k_data_size = atoi (argv[++i]);
-	else if (streq (argv [i], "-lkdata"))
-	  initial_k_data_limit = atoi (argv[++i]);
-	else
-	  error ("usage: spim -bare/-asm -trap/-notrap -trap_file <file> -quiet/-noquiet -mapped_io/-nomapped_io -file <file> <args>\n");
-      }
+	  if (!assembly_file_loaded)
+	    {
+	      initialize_world (load_exception_handler ? exception_file_name : NULL);
+	    }
+	  assembly_file_loaded |= !read_assembly_file (argv[++i]);
+	  break;
+	}
+      else if (argv [i][0] != '-')
+	{
+	  /* Assume this argument is a file name and everything following is
+	     arguments to program */
+	  program_argc = argc - i;
+	  program_argv = &argv[i]; /* Everything following is argv */
 
-  if (!assembly_file_read)
+	  if (!assembly_file_loaded)
+	    {
+	      initialize_world (load_exception_handler ? exception_file_name : NULL);
+	    }
+	  assembly_file_loaded |= !read_assembly_file (argv[i]);
+	  break;
+	}
+      else
+	{
+	  error ("\nUnknown argument: %s (ignored)\n", argv[i]);
+	  print_usage_msg = 1;
+	}
+    }
+
+  if (print_usage_msg)
     {
-      initialize_world (load_trap_handler ? trap_file : NULL);
+      error ("Usage: spim\n\
+	-bare			Bare machine (no pseudo-ops, delayed branches and loads)\n\
+	-asm			Extended machine (pseudo-ops, no delayed branches and loads) (default)\n\
+	-delayed_branches	Execute delayed branches\n\
+	-delayed_loads		Execute delayed loads\n\
+	-exception		Load exception handler (default)\n\
+	-noexception		Do not load exception handler\n\
+	-exception_file <file>	Specify exception handler in place of default\n\
+	-quiet			Do not print warnings\n\
+	-noquiet		Print warnings (default)\n\
+	-mapped_io		Enable memory-mapped IO\n\
+	-nomapped_io		Do not enable memory-mapped IO (default)\n\
+	-file <file> <args>	Assembly code file and arguments to program\n");
+    }
+
+  if (!assembly_file_loaded)
+    {
+      initialize_world (load_exception_handler ? exception_file_name : NULL);
       top_level ();
     }
-  else /* assembly_file_read */
+  else /* assembly_file_loaded */
     {
       console_to_program ();
-      initialize_run_stack (argc - argv_ptr, &argv[argv_ptr]);
+      initialize_run_stack (program_argc, program_argv);
       if (!setjmp (spim_top_level_env))
 	{
 	  char *undefs = undefined_symbol_string ();
@@ -288,23 +319,19 @@ main (argc, argv)
       console_to_spim ();
     }
 
-  return (0);
+  spim_return_value = 0;
+  return (spim_return_value);
 }
 
 
 /* Top-level read-eval-print loop for SPIM. */
 
-#ifdef __STDC__
-static void
-top_level (void)
-#else
 static void
 top_level ()
-#endif
 {
   int redo = 0;			/* Non-zero means reexecute last command */
 
-  signal (SIGINT, control_c_seen);
+  (void)signal (SIGINT, control_c_seen);
   while (1)
     {
       if (!redo)
@@ -319,14 +346,8 @@ top_level ()
 }
 
 
-#ifdef __STDC__
 void
 control_c_seen (int arg)
-#else
-void
-control_c_seen (arg)
-int arg;
-#endif
 {
   console_to_spim ();
   write_output (message_out, "\nExecution interrupted\n");
@@ -362,15 +383,8 @@ enum {
    don't read a new command; just rexecute the previous one.
    Return non-zero if the command was to redo the previous command. */
 
-#ifdef __STDC__
 static int
 parse_spim_command (FILE *file, int redo)
-#else
-static int
-parse_spim_command (file, redo)
-     FILE *file;
-     int redo;
-#endif
 {
   static int prev_cmd = NOP_CMD; /* Default redo */
   static int prev_token;
@@ -408,9 +422,9 @@ parse_spim_command (file, redo)
 	if (addr == 0)
 	  addr = starting_address ();
 
-	initialize_run_stack (0, 0);
+	initialize_run_stack (program_argc, program_argv);
 	console_to_program ();
-	if (addr)
+	if (addr != 0)
 	{
 	  char *undefs = undefined_symbol_string ();
 	  if (undefs != NULL)
@@ -455,7 +469,7 @@ parse_spim_command (file, redo)
 
 	if (steps == 0)
 	  steps = 1;
-	if (addr)
+	if (addr != 0)
 	  {
 	    console_to_program ();
 	    if (run_program (addr, steps, 1, 1))
@@ -532,7 +546,7 @@ parse_spim_command (file, redo)
 
     case REINITIALIZE_CMD:
       flush_to_newline ();
-      initialize_world (load_trap_handler ? trap_file : NULL);
+      initialize_world (load_exception_handler ? exception_file_name : NULL);
       write_startup_message ();
       prev_cmd = NOP_CMD;
       return (0);
@@ -665,14 +679,14 @@ parse_spim_command (file, redo)
           {
             int32 code = inst_encode (text_seg[i]);
             if (cmd == DUMP_TEXT_CMD)
-	      code = htonl (code);    /* dump in network byte order */
-            fwrite (&code, 1, sizeof(code), fp);
+	      code = (int32)htonl ((unsigned long)code);    /* dump in network byte order */
+            (void)fwrite (&code, 1, sizeof(code), fp);
             words++;
           }
 
         fclose (fp);
-        fprintf (stderr, "Dumped %ld words starting at 0x%08lx to file %s\n",
-                 words, (dump_start << 2) + TEXT_BOT, filename);
+        fprintf (stderr, "Dumped %d words starting at 0x%08x to file %s\n",
+                 words, (unsigned int)((dump_start << 2) + TEXT_BOT), filename);
       }
       prev_cmd = cmd;
       return (0);
@@ -688,13 +702,8 @@ parse_spim_command (file, redo)
 /* Read a SPIM command with the scanner and return its ennuemerated
    value. */
 
-#ifdef __STDC__
-static int
-read_assembly_command (void)
-#else
 static int
 read_assembly_command ()
-#endif
 {
   int token = read_token ();
 
@@ -747,15 +756,8 @@ read_assembly_command ()
 
 /* Return non-nil if STRING1 is a (proper) prefix of STRING2. */
 
-#ifdef __STDC__
 static int
 str_prefix (char *s1, char *s2, int min_match)
-#else
-static int
-str_prefix (s1, s2, min_match)
-     char *s1, *s2;
-     int min_match;
-#endif
 {
   for ( ; *s1 == *s2 && *s1 != '\0'; s1 ++, s2 ++) min_match --;
   return (*s1 == '\0' && min_match <= 0);
@@ -766,13 +768,8 @@ str_prefix (s1, s2, min_match)
    line doesn't contain an integer, return 0.  In either case, flush the
    rest of the line, including the newline. */
 
-#ifdef __STDC__
-static int
-get_opt_int (void)
-#else
 static int
 get_opt_int ()
-#endif
 {
   int token;
 
@@ -793,13 +790,8 @@ get_opt_int ()
 
 /* Flush the rest of the input line up to and including the next newline. */
 
-#ifdef __STDC__
-static void
-flush_to_newline (void)
-#else
 static void
 flush_to_newline ()
-#endif
 {
   while (read_token () != Y_NL) ;
 }
@@ -808,28 +800,16 @@ flush_to_newline ()
 /* Print register number N.
    Return non-zero if register N was valid register string. */
 
-#ifdef __STDC__
 static int
 print_reg (int reg_no)
-#else
-static int
-print_reg (reg_no)
-     int reg_no;
-#endif
 {
   write_output (message_out, "Reg %d = 0x%08x (%d)\n", reg_no, R[reg_no], R[reg_no]);
   return (1);
 }
 
 
-#ifdef __STDC__
 static int
 print_fp_reg (int reg_no)
-#else
-static int
-print_fp_reg (reg_no)
-     int reg_no;
-#endif
 {
   if ((reg_no & 1) == 0)
     write_output (message_out, "FP reg %d = %g (double)\n", reg_no, FPR_D (reg_no));
@@ -838,14 +818,8 @@ print_fp_reg (reg_no)
 }
 
 
-#ifdef __STDC__
 static int
 print_reg_from_string (char* reg_num)
-#else
-static int
-print_reg_from_string (reg_num)
-     char* reg_num;
-#endif
 {
   char s[100];
   char *s1 = s;
@@ -867,22 +841,15 @@ print_reg_from_string (reg_num)
   else if (streq (s1, "lo"))
     write_output (message_out, "LO = 0x%08x (%d)\n", LO, LO);
   else if (streq (s1, "fpcond"))
-    write_output (message_out, "FpCond = 0x%08x (%d)\n", FpCond, FpCond);
+    write_output (message_out, "FCSR = 0x%08x (%d)\n", FCSR, FCSR);
   else if (streq (s1, "cause"))
-    write_output (message_out, "Cause = 0x%08x (%d)\n", Cause, Cause);
+    write_output (message_out, "Cause = 0x%08x (%d)\n", CP0_Cause, CP0_Cause);
   else if (streq (s1, "epc"))
-    write_output (message_out, "EPC = 0x%08x (%d)\n", EPC, EPC);
+    write_output (message_out, "EPC = 0x%08x (%d)\n", CP0_EPC, CP0_EPC);
   else if (streq (s1, "status"))
-    write_output (message_out, "Status = 0x%08x (%d)\n",
-		  Status_Reg, Status_Reg);
+    write_output (message_out, "Status = 0x%08x (%d)\n", CP0_Status, CP0_Status);
   else if (streq (s1, "badvaddr"))
-    write_output (message_out, "BadVAddr = 0x%08x (%d)\n",
-		  BadVAddr, BadVAddr);
-  else if (streq (s1, "context"))
-    write_output (message_out, "Context = 0x%08x (%d)\n",
-		  Context, Context);
-  else if (streq (s1, "prid"))
-    write_output (message_out, "PRId = 0x%08x (%d)\n", PRId, PRId);
+    write_output (message_out, "BadVAddr = 0x%08x (%d)\n", CP0_BadVAddr, CP0_BadVAddr);
   else
     return (0);
 
@@ -890,49 +857,26 @@ print_reg_from_string (reg_num)
 }
 
 
-#define MAX_BUF_LEN 32000
-
-#ifdef __STDC__
 static void
 print_all_regs (int hex_flag)
-#else
-static void
-print_all_regs (hex_flag)
-     int hex_flag;
-#endif
 {
-  int max_buf_len = MAX_BUF_LEN;
-  char buf[MAX_BUF_LEN];
-  int count;
-  registers_as_string (buf, &max_buf_len, &count, hex_flag, hex_flag);
-  write_output (message_out, "%s\n", buf);
+  static str_stream ss;
+
+  ss_clear (&ss);
+  format_registers (&ss, hex_flag, hex_flag);
+  write_output (message_out, "%s\n", ss_to_string (&ss));
 }
 
 
 
 /* Print an error message. */
 
-#ifdef __STDC__
 void
 error (char *fmt, ...)
-#else
-/*VARARGS0*/
-void
-error (va_alist)
-va_dcl
-#endif
 {
   va_list args;
-#ifndef __STDC__
-  char *fmt;
-#endif
 
-#ifdef __STDC__
   va_start (args, fmt);
-#else
-  va_start (args);
-  fmt = va_arg (args, char *);
-#endif
 
 #ifdef NO_VFPRINTF
   _doprnt (fmt, args, stderr);
@@ -943,73 +887,57 @@ va_dcl
 }
 
 
-/* Print an error message and return to top level. */
+/* Print the error message then exit. */
 
-#ifdef __STDC__
-int*
-run_error (char *fmt, ...)
-#else
-/*VARARGS0*/
-int*
-run_error (va_alist)
-va_dcl
-#endif
+void
+fatal_error (char *fmt, ...)
 {
   va_list args;
-#ifndef __STDC__
-  char *fmt;
-#endif
-
-#ifdef __STDC__
   va_start (args, fmt);
-#else
-  va_start (args);
   fmt = va_arg (args, char *);
+
+#ifdef NO_VFPRINTF
+  _doprnt (fmt, args, stderr);
+#else
+  vfprintf (stderr, fmt, args);
 #endif
+  exit (-1);
+}
+
+
+/* Print an error message and return to top level. */
+
+void
+run_error (char *fmt, ...)
+{
+  va_list args;
+
+  va_start (args, fmt);
 
   console_to_spim ();
 
-#ifdef VFPRINTF
+#ifdef NO_VFPRINTF
   _doprnt (fmt, args, stderr);
 #else
   vfprintf (stderr, fmt, args);
 #endif
   va_end (args);
   longjmp (spim_top_level_env, 1);
-  return (0);			/* So it can be used in expressions */
 }
 
 
 
 /* IO facilities: */
 
-#ifdef __STDC__
 void
 write_output (port fp, char *fmt, ...)
-#else
-/*VARARGS0*/
-void
-write_output (va_alist)
-va_dcl
-#endif
 {
   va_list args;
   FILE *f;
-#ifndef __STDC__
-  char *fmt;
-  port fp;
-#endif
   int restore_console_to_program = 0;
 
-#ifdef __STDC__
   va_start (args, fmt);
-  f = (FILE *) fp.f;
-#else
-  va_start (args);
-  fp = va_arg (args, port);
-  f = (FILE *) fp.f;
-  fmt = va_arg (args, char *);
-#endif
+  f = fp.f;
 
   if (console_state_saved)
     {
@@ -1044,15 +972,8 @@ va_dcl
 
 /* Simulate the semantics of fgets (not gets) on Unix file. */
 
-#ifdef __STDC__
 void
 read_input (char *str, int str_size)
-#else
-void
-read_input (str, str_size)
-     char *str;
-     int str_size;
-#endif
 {
   char *ptr;
   int restore_console_to_program = 0;
@@ -1087,32 +1008,17 @@ read_input (str, str_size)
 
 /* Give the console to the program for IO. */
 
-#ifdef __STDC__
-static void
-console_to_program (void)
-#else
 static void
 console_to_program ()
-#endif
 {
   if (mapped_io && !console_state_saved)
     {
-#ifdef USE_TERMIO
-      struct termio params;
+#ifdef USE_TERMIOS
+      struct termios params;
 
-#ifdef DJGPP
-      tcgetattr((int)console_in.i, &saved_console_state);
-      params = saved_console_state;
-
-      /* Note:  You must include the ICRNL option on DOS
-		IUCLC and IXANY simply are not supported under DJGPP */
-      params.c_iflag &= ~(ISTRIP|INLCR|IGNCR|IXON|IXOFF|INPCK|BRKINT|PARMRK);
-      params.c_iflag |= ICRNL;
-#else
       ioctl (console_in.i, TCGETA, (char *) &saved_console_state);
       params = saved_console_state;
       params.c_iflag &= ~(ISTRIP|IUCLC|INLCR|ICRNL|IGNCR|IXON|IXOFF|IXANY|INPCK|BRKINT|PARMRK);
-#endif
 
       params.c_iflag |= IGNBRK|IGNPAR;
       /*params.c_oflag &= ~OPOST;*/
@@ -1121,11 +1027,7 @@ console_to_program ()
       params.c_lflag = 0;
       params.c_cc[VMIN] = 1;
       params.c_cc[VTIME] = 1;
-#ifdef DJGPP
-      tcsetattr((int)console_in.i, TCSANOW, &params);
-#else
       ioctl ((int)console_in.i, TCSETA, (char *) &params);
-#endif
 #else
       int flags;
       ioctl ((int) console_in.i, TIOCGETP, (char *) &saved_console_state);
@@ -1141,21 +1043,12 @@ console_to_program ()
 
 /* Return the console to SPIM. */
 
-#ifdef __STDC__
-static void
-console_to_spim (void)
-#else
 static void
 console_to_spim ()
-#endif
 {
   if (mapped_io && console_state_saved)
-#ifdef USE_TERMIO
-#ifdef DJGPP
-    tcsetattr((int)console_in.i, &saved_console_state);
-#else
+#ifdef USE_TERMIOS
     ioctl ((int) console_in.i, TCSETA, (char *) &saved_console_state);
-#endif
 #else
     ioctl ((int) console_in.i, TIOCSETP, (char *) &saved_console_state);
 #endif
@@ -1163,15 +1056,9 @@ console_to_spim ()
 }
 
 
-#ifdef __STDC__
-int
-console_input_available (void)
-#else
 int
 console_input_available ()
-#endif
 {
-#ifndef __CYGWIN32__
   fd_set fdset;
   struct timeval timeout;
 
@@ -1184,18 +1071,12 @@ console_input_available ()
       return (select (sizeof (fdset) * 8, &fdset, NULL, NULL, &timeout));
     }
   else
-#endif
     return (0);
 }
 
 
-#ifdef __STDC__
-char
-get_console_char (void)
-#else
 char
 get_console_char ()
-#endif
 {
   char buf;
 
@@ -1207,26 +1088,15 @@ get_console_char ()
 }
 
 
-#ifdef __STDC__
 void
 put_console_char (char c)
-#else
-void
-put_console_char (c)
-     char c;
-#endif
 {
-  putc (c, (FILE *) console_out.f);
-  fflush ((FILE *) console_out.f);
+  putc (c, console_out.f);
+  fflush (console_out.f);
 }
 
-#ifdef __STDC__
 static int
 read_token ()
-#else
-static int
-read_token ()
-#endif
 {
   int token = yylex ();
 
