@@ -2,7 +2,7 @@
    X interface to SPIM
    (Derived from an earlier work by Alan Siow.)
 
-   Copyright (C) 1990-2000 by James Larus (larus@cs.wisc.edu).
+   Copyright (C) 1990-2004 by James Larus (larus@cs.wisc.edu).
    ALL RIGHTS RESERVED.
 
    SPIM is distributed under the following conditions:
@@ -21,16 +21,12 @@
    PURPOSE. */
 
 
-/* $Header: /Software/SPIM/src/xspim.c 10    1/01/03 9:34p Larus $
+/* $Header: /Software/SPIM/src/xspim.c 26    11/26/04 8:47p Larus $
  */
 
 #include <stdio.h>
 #include <setjmp.h>
-#ifdef __STDC__
 #include <stdarg.h>
-#else
-#include <varargs.h>
-#endif
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -44,10 +40,11 @@
 #include <X11/keysym.h>
 
 #include "spim.h"
+#include "string-stream.h"
 #include "spim-utils.h"
 #include "inst.h"
-#include "mem.h"
 #include "reg.h"
+#include "mem.h"
 #include "y.tab.h"
 #include "buttons.h"
 #include "windows.h"
@@ -63,12 +60,11 @@ typedef struct _AppResources
   Boolean delayed_loads;
   Boolean pseudo;
   Boolean asmm;
-  Boolean trap;
-  char *trap_file;
+  Boolean exception;
+  char *exception_file_name;
   Boolean quiet;
   Boolean mapped_io;
   char *filename;
-  char *ex_filename;
   char *display2;
   Boolean hex_gpr;
   Boolean hex_fpr;
@@ -87,71 +83,44 @@ typedef struct _AppResources
 
 /* Not local, but not export so all files don't need setjmp.h */
 jmp_buf spim_top_level_env; /* For ^C */
-reg_word R[32];
-reg_word HI, LO;
-int HI_present, LO_present;
-mem_addr PC, nPC;
-double *FPR;			/* Dynamically allocate so overlay */
-float *FGR;			/* is possible */
-int *FWR;			/* is possible */
-int FP_reg_present;		/* Presence bits for FP registers */
-int FP_reg_poison;		/* Poison bits for FP registers */
-int FP_spec_load;		/* Is register waiting for a speculative ld */
-reg_word CpCond[4], CCR[4][32], CPR[4][32];
 
 int bare_machine;		/* Non-Zero => simulate bare machine */
 int delayed_branches;		/* Non-Zero => simulate delayed branches */
 int delayed_loads;		/* Non-Zero => simulate delayed loads */
 int accept_pseudo_insts;	/* Non-Zero => parse pseudo instructions  */
 int quiet;			/* Non-Zero => no warning messages */
-int source_file;		/* Non-Zero => program is source, not binary */
 port message_out, console_out, console_in;
 int mapped_io;			/* Non-zero => activate memory-mapped IO */
 int pipe_out;
-int cycle_level;		/* Non-zero => cycle level mode */
+int spim_return_value;		/* Value returned when spim exits */
 
 XtAppContext app_con;
 Widget message, console = NULL;
 XtAppContext app_context;
 XFontStruct *text_font;
 Dimension button_width;
-int load_trap_handler;
-char *trap_file = DEFAULT_TRAP_HANDLER;
+int load_exception_handler;
+char *exception_file_name = DEFAULT_EXCEPTION_HANDLER;
 Pixmap mark;
 
 
 /* Local functions: */
 
-#ifdef __STDC__
-static void center_text_at_PC (void);
+static void center_text_at_PC ();
 static char *check_buf_limit (char *, int *, int *);
-static void create_console_display (void);
-static void display_data_seg (void);
+static void create_console_display ();
+static void display_data_seg ();
 static char *display_values (mem_addr from, mem_addr to, char *buf, int *limit,
 			     int *n);
 static char *display_insts (mem_addr from, mem_addr to, char *buf, int *limit,
 			    int *n);
-static void display_registers (void);
+static void display_registers ();
 static void initialize (AppResources app_res);
 static mem_addr print_partial_line (mem_addr, char *, int *, int *);
-static void show_running (void);
+static void show_running ();
 static void syntax (char *program_name);
 static void write_text_to_window (Widget w, char *s);
 
-#else
-static void center_text_at_PC ();
-static char *check_buf_limit ();
-static void create_console_display ();
-static void display_data_seg ();
-static char *display_values ();
-static char *display_insts ();
-static void display_registers ();
-static void initialize ();
-static mem_addr print_partial_line ();
-static void show_running ();
-static void syntax ();
-static void write_text_to_window ();
-#endif
 
 static String fallback_resources[] =
 {
@@ -192,10 +161,10 @@ static XtResource resources[] =
    XtOffset (AppResources *, pseudo), XtRImmediate, (XtPointer)True},
   {"asm",  "Asm",  XtRBoolean, sizeof (Boolean),
    XtOffset (AppResources *, asmm), XtRImmediate, False},
-  {"trap", "Trap", XtRBoolean, sizeof (Boolean),
-   XtOffset (AppResources *, trap), XtRImmediate, (XtPointer) True},
-  {"trap_file", "Trap_File", XtRString, sizeof (char *),
-   XtOffset (AppResources *, trap_file), XtRString, NULL},
+  {"exception", "Exception", XtRBoolean, sizeof (Boolean),
+   XtOffset (AppResources *, exception), XtRImmediate, (XtPointer) True},
+  {"exception_file_name", "Exception_File_Name", XtRString, sizeof (char *),
+   XtOffset (AppResources *, exception_file_name), XtRString, NULL},
   {"quiet", "Quiet", XtRBoolean, sizeof (Boolean),
    XtOffset (AppResources *, quiet), XtRImmediate, False},
   {"mapped_io", "Mapped_IO", XtRBoolean, sizeof (Boolean),
@@ -203,8 +172,6 @@ static XtResource resources[] =
 
   {"filename", "Filename", XtRString, sizeof (char *),
    XtOffset (AppResources *, filename), XtRString, NULL},
-  {"ex_filename", "Ex_Filename", XtRString, sizeof (char *),
-   XtOffset (AppResources *, ex_filename), XtRString, NULL},
   {"display2", "Display2", XtRString, sizeof (char *),
    XtOffset (AppResources *, display2), XtRString, NULL},
   {"hexGpr", "DisplayHex", XtRBoolean, sizeof (Boolean),
@@ -233,35 +200,61 @@ static XtResource resources[] =
 
 static XrmOptionDescRec options[] =
 {
-  {"-bare",   "bare", XrmoptionNoArg, "True"},
   {"-asm",    "asmm",  XrmoptionNoArg, "True"},
-  {"-pseudo",   "pseudo", XrmoptionNoArg, "True"},
+  {"-a",    "asmm",  XrmoptionNoArg, "True"},
+  {"-bare",   "bare", XrmoptionNoArg, "True"},
+  {"-b",   "bare", XrmoptionNoArg, "True"},
   {"-delayed_branches", "delayed_branches", XrmoptionNoArg, "True"},
+  {"-db", "delayed_branches", XrmoptionNoArg, "True"},
   {"-delayed_loads", "delayed_loads", XrmoptionNoArg, "True"},
-  {"-nopseudo", "pseudo", XrmoptionNoArg, "False"},
-  {"-trap",   "trap", XrmoptionNoArg, "True"},
-  {"-notrap", "trap", XrmoptionNoArg, "False"},
-  {"-trap_file", "trap_file", XrmoptionSepArg, NULL},
-  {"-quiet",  "quiet", XrmoptionNoArg, "True"},
-  {"-noquiet","quiet", XrmoptionNoArg, "False"},
+  {"-dl", "delayed_loads", XrmoptionNoArg, "True"},
+  {"-exception",   "exception", XrmoptionNoArg, "True"},
+  {"-e",   "exception", XrmoptionNoArg, "True"},
+  {"-noexception", "exception", XrmoptionNoArg, "False"},
+  {"-ne", "exception", XrmoptionNoArg, "False"},
+  {"-exception_file_name", "exception_file_name", XrmoptionSepArg, NULL},
+  {"-ef", "exception_file_name", XrmoptionSepArg, NULL},
   {"-mapped_io",  "mapped_io", XrmoptionNoArg, "True"},
+  {"-mio",  "mapped_io", XrmoptionNoArg, "True"},
   {"-nomapped_io","mapped_io", XrmoptionNoArg, "False"},
-
+  {"-nmio","mapped_io", XrmoptionNoArg, "False"},
+  {"-pseudo",   "pseudo", XrmoptionNoArg, "True"},
+  {"-p",   "pseudo", XrmoptionNoArg, "True"},
+  {"-nopseudo", "pseudo", XrmoptionNoArg, "False"},
+  {"-np", "pseudo", XrmoptionNoArg, "False"},
+  {"-quiet",  "quiet", XrmoptionNoArg, "True"},
+  {"-q",  "quiet", XrmoptionNoArg, "True"},
+  {"-noquiet","quiet", XrmoptionNoArg, "False"},
+  {"-nq","quiet", XrmoptionNoArg, "False"},
+  {"-trap",   "exception", XrmoptionNoArg, "True"},
+  {"-t",   "exception", XrmoptionNoArg, "True"},
+  {"-notrap", "exception", XrmoptionNoArg, "False"},
+  {"-nt", "exception", XrmoptionNoArg, "False"},
+  {"-trap_file_name", "exception_file_name", XrmoptionSepArg, NULL},
+  {"-tf", "exception_file_name", XrmoptionSepArg, NULL},
+  {"-stext", "stext", XrmoptionSepArg, NULL},
+  {"-st", "stext", XrmoptionSepArg, NULL},
+  {"-sdata", "sdata", XrmoptionSepArg, NULL},
+  {"-sd", "sdata", XrmoptionSepArg, NULL},
+  {"-ldata", "ldata", XrmoptionSepArg, NULL},
+  {"-ld", "ldata", XrmoptionSepArg, NULL},
+  {"-sstack", "sstack", XrmoptionSepArg, NULL},
+  {"-ss", "sstack", XrmoptionSepArg, NULL},
+  {"-lstack", "lstack", XrmoptionSepArg, NULL},
+  {"-ls", "lstack", XrmoptionSepArg, NULL},
+  {"-sktext", "sktext", XrmoptionSepArg, NULL},
+  {"-skt", "sktext", XrmoptionSepArg, NULL},
+  {"-skdata", "skdata", XrmoptionSepArg, NULL},
+  {"-skd", "skdata", XrmoptionSepArg, NULL},
+  {"-lkdata", "lkdata", XrmoptionSepArg, NULL},
+  {"-lkd", "lkdata", XrmoptionSepArg, NULL},
   {"-file",   "filename", XrmoptionSepArg, NULL},
-  {"-execute","ex_filename", XrmoptionSepArg, NULL},
+  {"-f",   "filename", XrmoptionSepArg, NULL},
   {"-d2",     "display2", XrmoptionSepArg, NULL},
   {"-hexgpr", "hexGpr", XrmoptionNoArg, "True"},
   {"-nohexgpr", "hexGpr", XrmoptionNoArg, "False"},
   {"-hexfpr", "hexFpr", XrmoptionNoArg, "True"},
   {"-nohexfpr", "hexFpr", XrmoptionNoArg, "False"},
-  {"-stext", "stext", XrmoptionSepArg, NULL},
-  {"-sdata", "sdata", XrmoptionSepArg, NULL},
-  {"-ldata", "ldata", XrmoptionSepArg, NULL},
-  {"-sstack", "sstack", XrmoptionSepArg, NULL},
-  {"-lstack", "lstack", XrmoptionSepArg, NULL},
-  {"-sktext", "sktext", XrmoptionSepArg, NULL},
-  {"-skdata", "skdata", XrmoptionSepArg, NULL},
-  {"-lkdata", "lkdata", XrmoptionSepArg, NULL}
 };
 
 
@@ -289,7 +282,6 @@ static Dimension command_hspace;
 static Dimension command_vspace;
 static int console_is_visible;
 static Dimension display_height;
-static char *ex_file_name = NULL;
 static char *file_name = NULL;
 static Dimension reg_min_height;
 static Dimension reg_max_height;
@@ -301,22 +293,14 @@ static Widget pane1;
 
 
 
-#ifdef __STDC__
 static void
 initialize (AppResources app_res)
-#else
-static void
-initialize (app_res)
-     AppResources app_res;
-#endif
 {
   bare_machine = 0;
   delayed_branches = 0;
   delayed_loads = 0;
   accept_pseudo_insts = 1;
   quiet = 0;
-  source_file = 0;
-  cycle_level = 0;
 
   if (app_res.bare)
     {
@@ -343,15 +327,15 @@ initialize (app_res)
   else
     accept_pseudo_insts = 0;
 
-  if (app_res.trap)
-    load_trap_handler = 1;
+  if (app_res.exception)
+    load_exception_handler = 1;
   else
-    load_trap_handler = 0;
+    load_exception_handler = 0;
 
-  if (app_res.trap_file)
+  if (app_res.exception_file_name)
     {
-      trap_file = app_res.trap_file;
-      load_trap_handler = 1;
+      exception_file_name = app_res.exception_file_name;
+      load_exception_handler = 1;
     }
 
   if (app_res.quiet)
@@ -366,11 +350,6 @@ initialize (app_res)
 
   if (app_res.filename)
     file_name = app_res.filename;
-  if (app_res.ex_filename)
-    {
-      ex_file_name = app_res.ex_filename;
-      load_trap_handler = 0;
-    }
 
   if (app_res.textFont == NULL)
     app_res.textFont = XtNewString ("8x13");
@@ -388,23 +367,18 @@ initialize (app_res)
     app_width = TEXTWIDTH * 4 * 22;
   command_hspace = 8;
   command_vspace = 8;
-  command_height = (button_height * 3) + (command_vspace * 4) + 2;
-  reg_min_height = 17 * TEXTHEIGHT + 4;
+  command_height = (button_height * 2) + (command_vspace * 3);
+  reg_min_height = 15 * TEXTHEIGHT + 4;
   reg_max_height = reg_min_height + 10 * TEXTHEIGHT + 4;
-  segment_height = 10 * TEXTHEIGHT + 4;
+  segment_height = 8 * TEXTHEIGHT + 4;
   display_height = 8 * TEXTHEIGHT + 4;
   print_gpr_hex = app_res.hex_gpr;
   print_fpr_hex = app_res.hex_fpr;
 }
 
 
-#ifdef __STDC__
-static void
-create_console_display (void)
-#else
 static void
 create_console_display ()
-#endif
 {
   Arg args[10];
   Cardinal n;
@@ -420,13 +394,8 @@ create_console_display ()
   console_out.f = (FILE*) console;
 }
 
-#ifdef __STDC__
-void
-clear_console_display (void)
-#else
 void
 clear_console_display ()
-#endif
 {
   Arg args[10];
   Cardinal n;
@@ -436,26 +405,21 @@ clear_console_display ()
 }
 
 
-#ifdef __STDC__
 int
 main (int argc, char **argv)
-#else
-int
-main (argc, argv)
-     int argc;
-     char **argv;
-#endif
 {
   Widget toplevel2;
   AppResources app_res;
   Display *display;
 
+  spim_return_value = 0;
+
   toplevel = XtAppInitialize (&app_context, "Xspim", options,
 			      XtNumber (options), &argc, argv,
 			      fallback_resources, NULL, ZERO);
 
-  if (argc != 1)
-    syntax (argv[0]);
+  if (argc >= 0 && argv[argc] != NULL && argv[argc][0] == '-')
+    syntax (argv[0]);		/* Bad command line argument */
 
   XtGetApplicationResources (toplevel, (XtPointer) &app_res, resources,
 			     XtNumber (resources), NULL, ZERO);
@@ -503,20 +467,21 @@ main (argc, argv)
     initial_k_data_size = atoi (app_res.initial_k_data_size);
   if (app_res.initial_k_data_limit != NULL)
     initial_k_data_limit = atoi (app_res.initial_k_data_limit);
+
   write_startup_message ();
-  initialize_world (load_trap_handler ? trap_file : NULL);
+  initialize_world (load_exception_handler ? exception_file_name : NULL);
+
+  if (file_name == NULL && argc >= 0 && argv[argc] != NULL)
+    {
+      /* First unprocessed argument */
+      file_name = argv[argc];
+    }
 
   if (file_name)
     {
-      read_file (file_name, 1);
+      read_file (file_name);
       record_file_name_for_prompt (file_name);
     }
-  else if (ex_file_name)
-    {
-      initialize_world (0);	/* Don't have a trap handler loaded. */
-      read_file (ex_file_name, 0);
-      record_file_name_for_prompt (ex_file_name);
-    }
   else
     {
       PC = starting_address ();
@@ -526,38 +491,34 @@ main (argc, argv)
     }
 
   XtAppMainLoop (app_context);
-  return (0);
+  return (spim_return_value);
 }
 
 
-#ifdef __STDC__
 static void
 syntax (char *program_name)
-#else
-static void
-syntax (program_name)
-     char *program_name;
-#endif
 {
   XtDestroyApplicationContext (app_context);
   fprintf (stderr, "Usage:\n %s", program_name);
-  fprintf (stderr, "\t[ -bare/-asm ] [ -trap/-notrap ] [ -quiet/noquiet ]\n");
-  fprintf (stderr, "\t[ -delayed_branches][ -delayed_loads]\n");
-  fprintf (stderr, "\t[ -pseudo/-nopseudo][ -mapped_io/-nomapped_io ]\n");
-  fprintf (stderr, "\t[ -d2 <display> ] [ -file/-execute <filename> ]\n");
-  fprintf (stderr, "\t[ -s<seg> <size>] [ -l<seg> <size>]\n");
+
+  fprintf (stderr, "-bare			Bare machine (no pseudo-ops, delayed branches and loads)\n");
+  fprintf (stderr, "-asm			Extended machine (pseudo-ops, no delayed branches and loads) (default)\n");
+  fprintf (stderr, "-delayed_branches	Execute delayed branches\n");
+  fprintf (stderr, "-delayed_loads		Execute delayed loads\n");
+  fprintf (stderr, "-exception		Load exception handler (default)\n");
+  fprintf (stderr, "-noexception		Do not load exception handler\n");
+  fprintf (stderr, "-exception_file <file>	Specify exception handler in place of default\n");
+  fprintf (stderr, "-quiet			Do not print warnings\n");
+  fprintf (stderr, "-noquiet		Print warnings (default)\n");
+  fprintf (stderr, "-mapped_io		Enable memory-mapped IO\n");
+  fprintf (stderr, "-nomapped_io		Do not enable memory-mapped IO (default)\n");
+  fprintf (stderr, "-file <file> <args>	Assembly code file and arguments to program\n");
   exit (1);
 }
 
 
-#ifdef __STDC__
 void
 control_c_seen (int arg)
-#else
-void
-control_c_seen (arg)
-int arg;
-#endif
 {
   write_output (message_out, "\nExecution interrupted\n");
   redisplay_data ();
@@ -567,15 +528,8 @@ int arg;
 }
 
 
-#ifdef __STDC__
 void
 popup_console (Widget w, XtPointer client_data, XtPointer call_data)
-#else
-void
-popup_console (w, client_data, call_data)
-     Widget w;
-     XtPointer client_data, call_data;
-#endif
 {
   if (console_is_visible)
     {
@@ -590,22 +544,16 @@ popup_console (w, client_data, call_data)
 }
 
 
-#ifdef __STDC__
 void
-read_file (char *name, int assembly_file)
-#else
-void
-read_file (name, assembly_file)
-     char *name;
-     int assembly_file;
-#endif
+read_file (char *name)
 {
   int error_flag;
 
   if (*name == '\0')
     error_flag = 1;
-  else if (assembly_file)
+  else
     error_flag = read_assembly_file (name);
+
   if (!error_flag)
     {
       PC = find_symbol_address (DEFAULT_RUN_LOCATION);
@@ -616,14 +564,8 @@ read_file (name, assembly_file)
 }
 
 
-#ifdef __STDC__
 void
 start_program (mem_addr addr)
-#else
-void
-start_program (addr)
-     mem_addr addr;
-#endif
 {
   if (addr == 0)
     addr = starting_address ();
@@ -633,15 +575,8 @@ start_program (addr)
 }
 
 
-#ifdef __STDC__
 void
 execute_program (mem_addr pc, int steps, int display, int cont_bkpt)
-#else
-void
-execute_program (pc, steps, display, cont_bkpt)
-     mem_addr pc;
-     int steps, display, cont_bkpt;
-#endif
 {
   if (!setjmp (spim_top_level_env))
     {
@@ -667,13 +602,8 @@ execute_program (pc, steps, display, cont_bkpt)
 }
 
 
-#ifdef __STDC__
-static void
-show_running (void)
-#else
 static void
 show_running ()
-#endif
 {
   Arg args[1];
 
@@ -685,13 +615,8 @@ show_running ()
 /* Redisplay the contents of the registers and, if modified, the data
    and stack segments. */
 
-#ifdef __STDC__
-void
-redisplay_data (void)
-#else
 void
 redisplay_data ()
-#endif
 {
   display_registers ();
   display_data_seg ();
@@ -701,56 +626,40 @@ redisplay_data ()
 /* Redisplay the contents of the registers in a wide variety of
    formats. */
 
-#ifdef __STDC__
-static void
-display_registers (void)
-#else
 static void
 display_registers ()
-#endif
 {
-  static char buf[8 * K];
-  int max_buf_len = 8 * K;
-  int string_len = 0;
   Arg args [2];
+  static str_stream ss;
 
-  registers_as_string (buf, &max_buf_len, &string_len, print_gpr_hex, print_fpr_hex);
+  ss_clear (&ss);
+  format_registers (&ss, print_gpr_hex, print_fpr_hex);
 
-  XtSetArg (args[0], XtNstring, (String)buf);
-  XtSetArg (args[1], XtNlength, string_len);
+  XtSetArg (args[0], XtNstring, (String)ss_to_string (&ss));
+  XtSetArg (args[1], XtNlength, ss_length (&ss));
   XtSetValues (register_window, args, TWO);
 }
 
 
 /* Redisplay the text segment and ktext segments if they have changed. */
 
-#ifdef __STDC__
-void
-redisplay_text (void)
-#else
 void
 redisplay_text ()
-#endif
 {
-  static String buf = NULL;
-  static int max_buf_len = 16 * K;
-  int string_len = 0;
   Arg args [2];
+  static str_stream ss;
 
   if (!text_modified)
     return;
 
-  if (buf == NULL)
-    buf = (String) malloc (max_buf_len);
-  *buf = '\0';
+  ss_clear (&ss);
 
-  buf = insts_as_string (TEXT_BOT, text_top, buf, &max_buf_len, &string_len);
-  sprintf (&buf[string_len], "\n\tKERNEL\n");
-  string_len += strlen (&buf[string_len]);
-  buf = insts_as_string (K_TEXT_BOT, k_text_top, buf, &max_buf_len, &string_len);
+  format_insts (&ss, TEXT_BOT, text_top);
+  ss_printf (&ss, "\n\tKERNEL\n");
+  format_insts (&ss, K_TEXT_BOT, k_text_top);
 
-  XtSetArg (args[0], XtNstring, buf);
-  XtSetArg (args[1], XtNlength, string_len);
+  XtSetArg (args[0], XtNstring, ss_to_string(&ss));
+  XtSetArg (args[1], XtNlength, ss_length (&ss));
   XtSetValues (text_window, args, TWO);
 
   text_modified = 0;
@@ -760,13 +669,8 @@ redisplay_text ()
 /* Center the text window at the instruction at the current PC and
    highlight the instruction. */
 
-#ifdef __STDC__
-static void
-center_text_at_PC (void)
-#else
 static void
 center_text_at_PC ()
-#endif
 {
   char buf[100];
   XawTextBlock text;
@@ -819,29 +723,20 @@ center_text_at_PC ()
 /* Display the contents of the data and stack segments, if they have
    been modified. */
 
-#ifdef __STDC__
-static void
-display_data_seg (void)
-#else
 static void
 display_data_seg ()
-#endif
 {
-  static String buf = NULL;
-  static int max_buf_len = 16 * K;
-  int string_len = 0;
   Arg args [2];
+  static str_stream ss;
 
   if (!data_modified)
     return;
 
-  if (buf == NULL)
-    buf = (char *) malloc (max_buf_len);
+  ss_clear (&ss);
+  format_data_segs(&ss);
 
-  buf = data_seg_as_string(buf, &max_buf_len, &string_len);
-
-  XtSetArg (args[0], XtNstring, buf);
-  XtSetArg (args[1], XtNlength, string_len);
+  XtSetArg (args[0], XtNstring, ss_to_string (&ss));
+  XtSetArg (args[1], XtNlength, ss_length (&ss));
   XtSetValues (data_window, args, TWO);
 
   data_modified = 0;
@@ -853,31 +748,14 @@ display_data_seg ()
 /* IO facilities: */
 
 
-#ifdef __STDC__
 void
 write_output (port fp, char *fmt, ...)
-#else
-/*VARARGS0*/
-void
-write_output (va_alist)
-va_dcl
-#endif
 {
   va_list args;
   Widget w;
-#ifndef __STDC__
-  char *fmt;
-  port fp;
-#endif
   char io_buffer [IO_BUFFSIZE];
 
-#ifdef __STDC__
   va_start (args, fmt);
-#else
-  va_start (args);
-  fp = va_arg (args, port);
-  fmt = va_arg (args, char *);
-#endif
   w = (Widget) fp.f;
 
   if (w == console && !console_is_visible)
@@ -904,15 +782,8 @@ va_dcl
 
 /* Simulate the semantics of fgets, not gets, on an x-window. */
 
-#ifdef __STDC__
 void
 read_input (char *str, int str_size)
-#else
-void
-read_input (str, str_size)
-     char *str;
-     int str_size;
-#endif
 {
   char buffer[11];
   KeySym key;
@@ -964,28 +835,16 @@ read_input (str, str_size)
 }
 
 
-#ifdef __STDC__
-int
-console_input_available (void)
-#else
+/* Checking for console input is messy with X windows. A key stroke appears
+   as an X event, which must be read to check its type. In addition, the
+   functions that read events all block.  So, when we check if console input
+   is available, we may see an event for a keystroke. If so, save the
+   keystroke for the next time the program reads a character. */
+
+static int previous_char = -1;
+
 int
 console_input_available ()
-#endif
-{
-  if (mapped_io)
-    return (XtAppPending (app_context));
-  else
-    return (0);
-}
-
-
-#ifdef __STDC__
-char
-get_console_char (void)
-#else
-char
-get_console_char ()
-#endif
 {
   XEvent event;
 
@@ -995,35 +854,42 @@ get_console_char ()
       console_is_visible = 1;
     }
 
-  while (1)
+  if (XtAppPending (app_context) != 0)
     {
+      /* There is an event to process: */
       XtAppNextEvent (app_context, &event);
       if (event.type == KeyPress)
 	{
+	  /* Event is a key stroke: */
 	  char buffer[11];
 	  KeySym key;
 	  XComposeStatus compose;
 	  XLookupString (&event.xkey, buffer, 10, &key, &compose);
 
-	  if (*buffer == 3)		       /* ^C */
+	  if (buffer[0] == 3)	/* ^C */
 	    XtDispatchEvent (&event);
-	  else if (*buffer != 0)
-	    return (buffer[0]);
+	  else if (buffer[0] != 0)
+	    previous_char = (int)buffer[0];
+	  return (1);		/* There is a character */
 	}
       else
-	XtDispatchEvent (&event);
+	{
+	  XtDispatchEvent (&event);
+	}
     }
+  return (0);    		/* There is no character */
+}
+
+char
+get_console_char ()
+{
+  /* assert (previous_char != -1); */
+  return (char)previous_char;
 }
 
 
-#ifdef __STDC__
 void
 put_console_char (char c)
-#else
-void
-put_console_char (c)
-     char c;
-#endif
 {
   char buf[4];
 
@@ -1041,28 +907,13 @@ put_console_char (c)
 
 /* Print an error message. */
 
-#ifdef __STDC__
 void
 error (char *fmt, ...)
-#else
-/*VARARGS0*/
-void
-error (va_alist)
-va_dcl
-#endif
 {
   va_list args;
-#ifndef __STDC__
-  char *fmt;
-#endif
   char io_buffer [IO_BUFFSIZE];
 
-#ifdef __STDC__
   va_start (args, fmt);
-#else
-  va_start (args);
-  fmt = va_arg (args, char *);
-#endif
   vsprintf (io_buffer, fmt, args);
   va_end (args);
   if (message != 0)
@@ -1072,28 +923,31 @@ va_dcl
 }
 
 
-#ifdef __STDC__
-int*
-run_error (char *fmt, ...)
-#else
-/*VARARGS0*/
-int*
-run_error (va_alist)
-va_dcl
-#endif
+/* Print the error message then exit. */
+
+void
+fatal_error (char *fmt, ...)
 {
   va_list args;
-#ifndef __STDC__
-  char *fmt;
+  va_start (args, fmt);
+  fmt = va_arg (args, char *);
+
+#ifdef NO_VFPRINTF
+  _doprnt (fmt, args, stderr);
+#else
+  vfprintf (stderr, fmt, args);
 #endif
+  exit (-1);
+}
+
+
+void
+run_error (char *fmt, ...)
+{
+  va_list args;
   char io_buffer [IO_BUFFSIZE];
 
-#ifdef __STDC__
   va_start (args, fmt);
-#else
-  va_start (args);
-  fmt = va_arg (args, char *);
-#endif
   vsprintf (io_buffer, fmt, args);
   va_end (args);
   if (message != 0)
@@ -1102,19 +956,11 @@ va_dcl
     fprintf (stderr, "%s", io_buffer);
   if (spim_is_running)
     longjmp (spim_top_level_env, 1);
-  return (0);			/* So it can be used in expressions */
 }
 
 
-#ifdef __STDC__
 static void
 write_text_to_window (Widget w, char *s)
-#else
-static void
-write_text_to_window (w, s)
-     Widget w;
-     char *s;
-#endif
 {
   XawTextBlock textblock;
   XawTextPosition ip = XawTextGetInsertionPoint (w);
